-- Localization
local L = LibStub("AceLocale-3.0"):GetLocale("TreasuresOfWoW")

-- Libraries
local config = LibStub("AceConfig-3.0")
local dialog = LibStub("AceConfigDialog-3.0")

local addon, TOW = ...
local abs, format, type, tostring, next, select, pairs, ipairs, string, table, tinsert = math.abs, format, type, tostring, next, select, pairs, ipairs, string, table, tinsert

TOW.OptionsName = L["Treasures Of WoW"]

local general = {
	order = 1,
	type = "group",
	name = TOW.OptionsName.." options :",
	args = {
		separator0 = {
		order = 1,
		name = "\n",
		type = "description",
		},
		DraenorTreasure = {
		order = 2,
		width = "full",
		name = L["ON/OFF "]..TOW.OptionsName,
		type = "toggle",
		set = function(info, val) TreasuresOfWoWDB.enabled = val end,
        get = function(info) return TreasuresOfWoWDB.enabled end,
		},
		separator1 = {
		order = 3,
		name = "\n",
		type = "description",
		},
		header1 = {
		order = 4,
		type = "header",
		name = L["Zones"],
		},
		separator2 = {
		order = 5,
		name = "\n",
		type = "description",
		},
		desc2 = {
		order = 6,
		type = "description",
		name = L["You can select the areas where you want to display the treasures."],
		},
		separator3 = {
		order = 7,
		name = "\n",
		type = "description",
		},
		Draenor = {
			name = L["Treasures Of Draenor"],
			order = 8,
			type = "group",
			inline = true,
			args = {
				mapIDhidden941 = {
				order = 8,
				name = L["Frostfire"],
				type = "toggle",
				set = function(info, val) TreasuresOfWoWDB.mapIDhidden[941] = val end,
				get = function(info) return TreasuresOfWoWDB.mapIDhidden[941] end,
				},
				mapIDhidden947 = {
				order = 9,
				name = L["Shadowmoon Valley"],
				type = "toggle",
				set = function(info, val) TreasuresOfWoWDB.mapIDhidden[947] = val end,
				get = function(info) return TreasuresOfWoWDB.mapIDhidden[947] end,
				},
				mapIDhidden949 = {
				order = 10,
				name = L["Gorgrond"],
				type = "toggle",
				set = function(info, val) TreasuresOfWoWDB.mapIDhidden[949] = val end,
				get = function(info) return TreasuresOfWoWDB.mapIDhidden[949] end,
				},
				mapIDhidden946 = {
				order = 11,
				name = L["Talador"],
				type = "toggle",
				set = function(info, val) TreasuresOfWoWDB.mapIDhidden[946] = val end,
				get = function(info) return TreasuresOfWoWDB.mapIDhidden[946] end,
				},
				mapIDhidden948 = {
				order = 12,
				name = L["Spires of Arak"],
				type = "toggle",
				set = function(info, val) TreasuresOfWoWDB.mapIDhidden[948] = val end,
				get = function(info) return TreasuresOfWoWDB.mapIDhidden[948] end,
				},
				mapIDhidden950 = {
				order = 13,
				name = L["Nagrand"],
				type = "toggle",
				set = function(info, val) TreasuresOfWoWDB.mapIDhidden[950] = val end,
				get = function(info) return TreasuresOfWoWDB.mapIDhidden[950] end,
				},
				mapIDhidden945 = {
				order = 14,
				name = L["Tanaan Jungle"],
				type = "toggle",
				set = function(info, val) TreasuresOfWoWDB.mapIDhidden[945] = val end,
				get = function(info) return TreasuresOfWoWDB.mapIDhidden[945] end,
				},
			},
		},
		Legion = {
			name = L["Treasures Of Legion"],
			order = 9,
			type = "group",
			inline = true,
			args = {
				mapIDhidden1015 = {
				order = 9,
				name = L["Azsuna"],
				type = "toggle",
				set = function(info, val) TreasuresOfWoWDB.mapIDhidden[1015] = val end,
				get = function(info) return TreasuresOfWoWDB.mapIDhidden[1015] end,
				},
				mapIDhidden1017 = {
				order = 10,
				name = L["Stormheim"],
				type = "toggle",
				set = function(info, val) TreasuresOfWoWDB.mapIDhidden[1017] = val end,
				get = function(info) return TreasuresOfWoWDB.mapIDhidden[1017] end,
				},
				mapIDhidden1018 = {
				order = 11,
				name = L["Val'sharah"],
				type = "toggle",
				set = function(info, val) TreasuresOfWoWDB.mapIDhidden[1018] = val end,
				get = function(info) return TreasuresOfWoWDB.mapIDhidden[1018] end,
				},
				mapIDhidden1024 = {
				order = 13,
				name = L["Highmountain"],
				type = "toggle",
				set = function(info, val) TreasuresOfWoWDB.mapIDhidden[1024] = val end,
				get = function(info) return TreasuresOfWoWDB.mapIDhidden[1024] end,
				},
				mapIDhidden1033 = {
				order = 14,
				name = L["Suramar"],
				type = "toggle",
				set = function(info, val) TreasuresOfWoWDB.mapIDhidden[1033] = val end,
				get = function(info) return TreasuresOfWoWDB.mapIDhidden[1033] end,
				},
			},
		},
		separator4 = {
		order = 15,
		name = "\n",
		type = "description",
		},
		header2 = {
		order = 20,
		type = "header",
		name = L["Minimap"],
		},
		separator20 = {
		order = 21,
		name = "\n",
		type = "description",
		},
		minimap = {
		order = 22,
		width = "full",
		name = L["ON/OFF display treasures on the minimap."],
		type = "toggle",
		set = function(info, val) TreasuresOfWoWDB.minimap = val end,
		get = function(info)	if TreasuresOfWoWDB.minimap then
									MinimapPOIHandler:Enable()
								else
									MinimapPOIHandler:Disable()
								end
								return TreasuresOfWoWDB.minimap end,
		},
		minimapAuto = {
		order = 23,
		width = "full",
		name = L["Reactive automatically displaying treasures."],
		desc = L["Reactive automatically display on the minimap treasure rebooting after each game if it was off."],
		type = "toggle",
		set = function(info, val) TreasuresOfWoWDB.minimapAuto = val end,
        get = function(info) return TreasuresOfWoWDB.minimapAuto end,
		},
		separator21 = {
		order = 24,
		name = "\n",
		type = "description",
		},
		header3 = {
		order = 30,
		type = "header",
		name = L["Others"],
		},
		separator30 = {
		order = 31,
		name = "\n",
		type = "description",
		},
		itm = {
		order = 32,
		width = "full",
		name = L["Displays treasures even if you bought the map of treasures."],
		type = "toggle",
		set = function(info, val) TreasuresOfWoWDB.ignoretm = val
			DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : treasure map quests now "]..(TreasuresOfWoWDB.ignoretm and L["ignored"] or L["used"]))
		end,
        get = function(info) return TreasuresOfWoWDB.ignoretm end,
		},
		arrow = {
		order = 33,
		width = "full",
		name = L["ON/OFF the arrow « Deadly Boss Mods »"],
		type = "toggle",
		disabled = function() return not DBM end,
		set = function(info, val) TOW:DBMArrow() TreasuresOfWoWDB.arrowEnabled = val end,
        get = function(info) return TreasuresOfWoWDB.arrowEnabled end,
		},
	},
}

local currentSelected, guiTreasures
local treasuresListe = {
	order = 2,
	type = "group",
	name = L["Treasures hidden options :"],
	args = {
		separator1 = {
		order = 1,
		name = "\n",
		type = "description",
		},
		addButton = {
			name = L["Add to Treasures hidden"],
			desc = L["Adds the nearest treasure our position has the hidden list."],
			order = 5,
			width = "full",
			type = "execute",
			func = function ()
				local closest, dist = GetClosestTreasure()
				if closest then
					TreasuresOfWoWDB.hidden[closest.id] = true
					DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : Treasure "]..closest.name.." ("..closest.id..") "..L["is now hidden"])
				else
					DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : there's no treasure nearby"])
				end
			end,
		},
		separator2 = {
		order = 10,
		name = "\n",
		type = "description",
		},
		trashlist = {
			name = L["Click a treasure below to remove it from the list hidden."],
			order = 15,
			type = "group",
			inline = true,
			args = {
            theList = {
				type = "multiselect",
				name = L["Treasures hidden"],
				width = "full",
				values = function () return TOW:GuiMakeTreasures() end,
				get = function () return currentSelected end,          
				set = function (info, key) TOW:GuiRemoveFromTreasures(key) end,
				},
			},
		},
		separator3 = {
		order = 20,
		name = "\n",
		type = "description",
		},
		clearTreasures = {
			name = L["Clear ALL Treasures Hidden"],
            order = 25,
			width = "full",
            type = "execute",
			disabled = function() return (not TreasuresOfWoWDB.hidden) or (not next(TreasuresOfWoWDB.hidden)) end,
            func = function() 
				table.wipe(TreasuresOfWoWDB.hidden)
				DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : hidden treasures cleared"]) 
			end,
        },
	},                                                            
}

local tomtom = {
	order = 3,
	type = "group",
	name = "Tomtom options :",
	args = {
		separator1 = {
		order = 1,
		name = "\n",
		type = "description",
		},
		desc1 = {
		order = 2,
		type = "description",
		name = L["This setting changes the modifier used by TomTom when left-clicking on the treasure map to create a waypoint"],
		},
		modifier = {
		type = "select",
		order = 3,
		name = L["Select buttom"],
		desc = L["This setting changes the modifier used by TomTom when left-clicking on the treasure map to create a waypoint"],
		set	= function(info, val) TreasuresOfWoWDB.tomtomKey = val end,
		get	= function(info) return TreasuresOfWoWDB.tomtomKey end,
		values	= function()
			return {
				["A"] = "Alt + left-click",
				["C"] = "Ctrl + left-click",
				["S"] = "Shift + left-click",
			} end,
		},
		separator2 = {
		order = 4,
		name = "\n",
		type = "description",
		},
		TomtomTreasures = {
			name = L["Go to the nearest treasure"],
            order = 5,
			width = "full",
            type = "execute",
            func = function() TOW:TomtomTreasure() end,
        },
	},
}

local debogage = {
	order = 4,
	type = "group",
	name = "Débogage options :",
	args = {
		separator4 = {
		order = 1,
		name = "\n",
		type = "description",
		},
		header3 = {
		order = 2,
		type = "header",
		name = "Débogage",
		},
		Debug = {
		order = 3,
		width = "full",
		name = L["ON/OFF débogage."],
		type = "toggle",
		set = function(info, val) TreasuresOfWoWDB.debug = val end,
        get = function(info) return TreasuresOfWoWDB.debug end,
		},
	},
}

function TOW:GuiMakeTreasures()
	local dbTreasures = TreasuresOfWoWDB.hidden
	guiTreasures = {}
	if (not dbTreasures) or (not next(dbTreasures)) then
		return guiTreasures
	else
		for i, v in pairs(TOW.treasures) do
			if dbTreasures[v.id] then
				guiTreasures[i] = L["Treasure ID : "]..v.id.." - "..v.name
			end
		end
		return guiTreasures
   end
end

function TOW:GuiRemoveFromTreasures(key)
	local treasureHidden = guiTreasures[key]
	if treasureHidden then
		for i, v in pairs(TOW.treasures) do
			local dbTreasures = TreasuresOfWoWDB.hidden
			local name = treasureHidden
			if (name and dbTreasures and name == L["Treasure ID : "]..v.id.." - "..v.name) then
				TreasuresOfWoWDB.hidden[v.id] = nil
				DEFAULT_CHAT_FRAME:AddMessage(L["Treasure ID : "]..v.id..L[" was deleted from the list."])
			end
		end
	end
end

local function findPanel(name, parent)
	for i, button in next, InterfaceOptionsFrameAddOns.buttons do
		if button.element then
			if name and button.element.name == name then return button
			elseif parent and button.element.parent == parent then return button
			end
		end
	end
end

function TOW:LoadOptions()
	if (config and dialog) then 
		-- Add options to Bliz addon config frame
		-- General Options
		config:RegisterOptionsTable(TOW.OptionsName, general)
		local options = dialog:AddToBlizOptions(TOW.OptionsName, TOW.OptionsName)
		options:HookScript("OnShow", function()
		local p = findPanel(TOW.OptionsName)
		if p and p.element.collapsed then OptionsListButtonToggle_OnClick(p.toggle) end
		end)
		
		-- Treasures List Options
		config:RegisterOptionsTable("TOW/treasuresListe", treasuresListe)
		dialog:AddToBlizOptions("TOW/treasuresListe", L["Treasures hidden"], TOW.OptionsName)
		
		-- TOMTOM Options
		if _G.TomTom then
			config:RegisterOptionsTable("TOW/tomtom", tomtom)
			dialog:AddToBlizOptions("TOW/tomtom", "Tomtom", TOW.OptionsName)
		end
		
		-- Debogage Options
		config:RegisterOptionsTable("TOW/debogage", debogage)
		dialog:AddToBlizOptions("TOW/debogage", "Débogage", TOW.OptionsName)
   end
end